/*******************************************************************************
 * Copyright (c) 2004, 2008 Etish Limited.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Barbara Rosi-Schwartz (Etish Limited, UK) - initial API and implementation
 *     Joel Rosi-Schwartz (Etish Limited, UK) - initial API and implementation
 *******************************************************************************/
package org.eclipse.ormf.server.base.services.domain;

import java.io.Serializable;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;
import javax.persistence.Version;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.eclipse.ormf.shared.util.logging.ILogger;
import org.eclipse.ormf.shared.util.logging.Logger;

@Entity(name = "Relationships")
@Table(name = "Relationships", schema = "useme", uniqueConstraints = @UniqueConstraint(columnNames = {
		"consumer_doc_id", "consumer_doc_type", "consumer_project_id",
		"provider_doc_id", "provider_doc_type", "provider_element_id",
		"provider_project_id", "stereotype" }))
public class Relationship implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3659273577544099068L;

	/**
	 * The instance of the logger for this class.
	 */
	private static final ILogger logger = Logger.Instance(Relationship.class);

	/** persistent field */
	private Long consumerDocId;

	/** persistent field */
	private Short consumerDocType;

	/** persistent field */
	private Short consumerProjectId;

	/** identifier field */
	private Long id;

	/** persistent field */
	private Boolean pendingAdjustment;

	/** persistent field */
	private Short pendingDelta;

	/** persistent field */
	private Long providerDocId;

	/** persistent field */
	private Short providerDocType;

	/** persistent field */
	private Short providerElementId;

	/** persistent field */
	private Short providerProjectId;

	/** persistent field */
	private Boolean providerRdbms;

	/** persistent field */
	private Short referenceCount;

	/** persistent field */
	private Short stereotype;

	/** nullable persistent field */
	private Integer version;

	/** default constructor */
	protected Relationship() {
	}

	/** full constructor */
	public Relationship(Long consumerDocId, Short consumerDocType,
			Short consumerProjectId, Long providerDocId, Short providerDocType,
			Short providerElementId, Short providerProjectId,
			Boolean providerRdbms, Short stereotype) {
		this.consumerDocId = consumerDocId;
		this.consumerDocType = consumerDocType;
		this.consumerProjectId = consumerProjectId;
		this.providerDocId = providerDocId;
		this.providerDocType = providerDocType;
		this.providerElementId = providerElementId;
		this.providerProjectId = providerProjectId;
		pendingAdjustment = true;
		pendingDelta = 1;
		this.providerRdbms = providerRdbms;
		referenceCount = 0;
		this.stereotype = stereotype;
	}

	public void adjustReferenceCount() {
		// TODO : JRS:I8 fix this kludge.
		referenceCount = Short.valueOf(String.valueOf(referenceCount
				+ pendingDelta));
		resetPendingAdjustment();
	}

	public void decrementReferenceCount() {
		pendingAdjustment = true;
		pendingDelta--;
	}

	@Override
	public boolean equals(Object other) {
		if ((this == other)) {
			return true;
		}

		if (!(other instanceof Relationship)) {
			return false;
		}

		final Relationship castOther = (Relationship) other;

		return new EqualsBuilder().append(getConsumerDocId(),
				castOther.getConsumerDocId()).append(getConsumerDocType(),
				castOther.getConsumerDocType()).append(getConsumerProjectId(),
				castOther.getConsumerProjectId()).append(getProviderDocId(),
				castOther.getProviderDocId()).append(getProviderDocType(),
				castOther.getProviderDocType()).append(getProviderElementId(),
				castOther.getProviderElementId()).append(
				getProviderProjectId(), castOther.getProviderProjectId())
				.append(getStereotype(), castOther.getStereotype()).isEquals();
	}

	@Transient
	public Integer getAdjustedReferenceCount() {
		return referenceCount + pendingDelta;
	}

	@Column(name = "consumer_doc_id", nullable = false, updatable = false)
	@Basic(optional = false)
	public Long getConsumerDocId() {
		return consumerDocId;
	}

	@Column(name = "consumer_doc_type", nullable = false, updatable = false)
	@Basic(optional = false)
	public Short getConsumerDocType() {
		return consumerDocType;
	}

	@Column(name = "consumer_project_id", nullable = false, updatable = false)
	@Basic(optional = false)
	public Short getConsumerProjectId() {
		return consumerProjectId;
	}

	@Id
	@Column(name = "id", updatable = false, nullable = false)
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	public Long getId() {
		return id;
	}

	@Column(name = "pending_delta", nullable = false)
	@Basic(optional = false)
	public Short getPendingDelta() {
		return pendingDelta;
	}

	@Column(name = "provider_doc_id", nullable = false, updatable = false)
	@Basic(optional = false)
	public Long getProviderDocId() {
		return providerDocId;
	}

	@Column(name = "provider_doc_type", nullable = false, updatable = false)
	@Basic(optional = false)
	public Short getProviderDocType() {
		return providerDocType;
	}

	@Column(name = "provider_element_id", nullable = false, updatable = false)
	@Basic(optional = false)
	public Short getProviderElementId() {
		return providerElementId;
	}

	@Column(name = "provider_project_id", nullable = false, updatable = false)
	@Basic(optional = false)
	public Short getProviderProjectId() {
		return providerProjectId;
	}

	@Column(name = "reference_count", nullable = false)
	@Basic(optional = false)
	public Short getReferenceCount() {
		return referenceCount;
	}

	@Column(nullable = false, updatable = false)
	@Basic(optional = false)
	public Short getStereotype() {
		return stereotype;
	}

	@Version
	protected Integer getVersion() {
		return version;
	}

	@Override
	public int hashCode() {
		return new HashCodeBuilder().append(getConsumerDocId()).append(
				getConsumerDocType()).append(getConsumerProjectId()).append(
				getProviderDocId()).append(getProviderDocType()).append(
				getProviderElementId()).append(getProviderProjectId()).append(
				getStereotype()).toHashCode();
	}

	public void incrementReferenceCount() {
		pendingAdjustment = true;
		pendingDelta++;
	}

	@Column(name = "pending_adjustment", nullable = false)
	@Basic(optional = false)
	public Boolean isPendingAdjustment() {
		return pendingAdjustment;
	}

	@Column(name = "provider_rdbms", nullable = false, updatable = false)
	@Basic(optional = false)
	public Boolean isProviderRdbms() {
		return providerRdbms;
	}

	/**
	 * Reset the pending adjustment flag. This has the effect that the
	 * outstanding pending adjustments will be set to zero and no adjustments
	 * will be made. This is used when the user has decided to throw away the
	 * changes to the document rather than save them.
	 */
	public void resetPendingAdjustment() {
		pendingAdjustment = false;
		pendingDelta = 0;
	}

	protected void setConsumerDocId(Long consumerDocId) {
		this.consumerDocId = consumerDocId;
	}

	protected void setConsumerDocType(Short consumerDocType) {
		this.consumerDocType = consumerDocType;
	}

	protected void setConsumerProjectId(Short consumerProjectId) {
		this.consumerProjectId = consumerProjectId;
	}

	protected void setId(Long id) {
		this.id = id;
	}

	protected void setPendingAdjustment(Boolean pendingAdjustment) {
		this.pendingAdjustment = pendingAdjustment;
	}

	protected void setPendingDelta(Short pendingDelta) {
		this.pendingDelta = pendingDelta;
	}

	protected void setProviderDocId(Long providerDocId) {
		this.providerDocId = providerDocId;
	}

	protected void setProviderDocType(Short providerDocType) {
		this.providerDocType = providerDocType;
	}

	protected void setProviderElementId(Short providerElementId) {
		this.providerElementId = providerElementId;
	}

	protected void setProviderProjectId(Short providerProjectId) {
		this.providerProjectId = providerProjectId;
	}

	protected void setProviderRdbms(Boolean providerRdbms) {
		this.providerRdbms = providerRdbms;
	}

	protected void setReferenceCount(Short referenceCount) {
		this.referenceCount = referenceCount;
	}

	protected void setStereotype(Short stereotype) {
		this.stereotype = stereotype;
	}

	protected void setVersion(Integer version) {
		this.version = version;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("id", getId()).append(
				"consumerDocId", getConsumerDocId()).append("consumerDocType",
				getConsumerDocType()).append("consumerProjectId",
				getConsumerProjectId()).append("providerDocId",
				getProviderDocId()).append("providerDocType",
				getProviderDocType()).append("providerElementId",
				getProviderElementId()).append("providerProjectId",
				getProviderProjectId()).append("stereotype", getStereotype())
				.toString();
	}
}