/*******************************************************************************
 * Copyright (c) 2004, 2008 Etish Limited.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Barbara Rosi-Schwartz (Etish Limited, UK) - initial API and implementation
 *     Joel Rosi-Schwartz (Etish Limited, UK) - initial API and implementation
 *******************************************************************************/
package org.eclipse.ormf.server.base.services.domain;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.Version;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.dom4j.Document;
import org.eclipse.ormf.shared.exceptions.LockViolationException;
import org.eclipse.ormf.shared.exceptions.ResourceException;
import org.eclipse.ormf.shared.model.dom4j.util.XmlHelper;
import org.eclipse.ormf.shared.util.logging.ILogger;
import org.eclipse.ormf.shared.util.logging.Logger;

@Entity(name = "ReqDocs")
@Table(name = "Req_Docs", schema = "useme")
public class ReqDoc implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 4468073653363870805L;

	/**
	 * The instance of the logger for this class.
	 */
	private static final ILogger logger = Logger.Instance(ReqDoc.class);

	/** persistent field */
	private Boolean hardLocked;

	/** identifier field */
	private Long id;

	/** persistent field */
	private Date lastUpdated;

	/** nullable persistent field */
	private User lockedBy;

	/**
	 * The the unique identifier in the Useme system for the project.
	 */
	private Short projectId;

	private Document rawDocument = null;

	/** persistent field */
	private Boolean softLocked;

	/**
	 * The type of the document.
	 */
	private Short type;

	/** nullable persistent field */
	private Integer version;

	/** persistent field */
	private String xml;

	/** default constructor */
	protected ReqDoc() {
	}

	/** full constructor */
	public ReqDoc(Short projectId, Short type, String xml) {
		assert projectId != null : "projectId may not be null";
		assert type != null : "type may not be null";

		hardLocked = false;
		lastUpdated = new Date();
		softLocked = false;

		this.projectId = projectId;
		this.type = type;
		this.xml = xml;
	}

	@Override
	public boolean equals(Object other) {
		if ((this == other)) {
			return true;
		}

		if (!(other instanceof ReqDoc)) {
			return false;
		}

		final ReqDoc castOther = (ReqDoc) other;

		return new EqualsBuilder().append(getId(), castOther.getId())
				.isEquals();
	}

	@Id
	@Column(name = "id", updatable = false, nullable = false)
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	public Long getId() {
		return id;
	}

	@Column(name = "last_updated", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	@Basic(optional = false)
	public Date getLastUpdated() {
		return lastUpdated;
	}

	@ManyToOne
	public User getLockedBy() {
		return lockedBy;
	}

	@Column(name = "project_id", nullable = false)
	@Basic(optional = false)
	public Short getProjectId() {
		return projectId;
	}

	@Transient
	public Document getRawDcoument() throws ResourceException {
		if (rawDocument == null) {
			try {
				rawDocument = XmlHelper.createDocument(xml);
			} catch (final Exception e) {
				throw new ResourceException(
						"Can not create document for reqdoc " + toString());
			}
		}

		return rawDocument;
	}

	@Column(nullable = false)
	@Basic(optional = false)
	public Short getType() {
		return type;
	}

	@Version
	protected Integer getVersion() {
		return version;
	}

	@Column(name = "the_xml", length = 512000, nullable = false)
	@Basic(optional = false)
	@Lob
	public String getXml() {
		return xml;
	}

	@Override
	public int hashCode() {
		return new HashCodeBuilder().append(getId()).toHashCode();
	}

	@Column(name = "hard_locked", nullable = false)
	@Basic(optional = false)
	public Boolean isHardLocked() {
		return hardLocked;
	}

	@Column(name = "soft_locked", nullable = false)
	@Basic(optional = false)
	public Boolean isSoftLocked() {
		return softLocked;
	}

	public void lockDoc(User user, Boolean hardlock, Boolean softlock)
			throws LockViolationException {
		if ((lockedBy != null) && !lockedBy.equals(user)) {
			throw new LockViolationException("The reqdoc[" + toString()
					+ "] is already locked by user[" + lockedBy + "]");
		}

		setLockedBy(user);
		setHardLocked(hardlock);
		setSoftLocked(softlock);
	}

	protected void setHardLocked(Boolean hardLocked) {
		this.hardLocked = hardLocked;
	}

	protected void setId(Long id) {
		this.id = id;
	}

	protected void setLastUpdated(Date lastUpdated) {
		this.lastUpdated = lastUpdated;
	}

	protected void setLockedBy(User lockedBy) {
		this.lockedBy = lockedBy;
	}

	protected void setProjectId(Short projectId) {
		this.projectId = projectId;
	}

	protected void setSoftLocked(Boolean softLocked) {
		this.softLocked = softLocked;
	}

	protected void setType(Short type) {
		this.type = type;
	}

	protected void setVersion(Integer version) {
		this.version = version;
	}

	public void setXml(String xml) {
		this.xml = xml;
		lastUpdated = new Date();
		rawDocument = null;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("id", getId()).append(
				"lockedBy", getLockedBy()).toString();
	}

	public void unlockDoc(User user) throws LockViolationException {
		if (!user.equals(lockedBy)) {
			throw new LockViolationException("The reqdoc[" + toString()
					+ "] is not locked by user[" + user + "]");
		}

		setSoftLocked(false);
		setHardLocked(false);
		lockedBy = null;
	}
}