/*******************************************************************************
 * Copyright (c) 2004, 2008 Etish Limited.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Barbara Rosi-Schwartz (Etish Limited, UK) - initial API and implementation
 *     Joel Rosi-Schwartz (Etish Limited, UK) - initial API and implementation
 *******************************************************************************/
package org.eclipse.ormf.server.base.services.domain;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Set;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.eclipse.ormf.shared.exceptions.AuthorizationException;
import org.eclipse.ormf.shared.exceptions.LockViolationException;
import org.eclipse.ormf.shared.exceptions.ProjectOpenInAnotherSessionException;
import org.eclipse.ormf.shared.util.logging.ILogger;
import org.eclipse.ormf.shared.util.logging.Logger;

@Entity(name = "Users")
@Table(name = "Users", schema = "useme")
public class User implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7421102968588011628L;

	/**
	 * The instance of the logger for this class.
	 */
	private static final ILogger logger = Logger.Instance(User.class);

	/** persistent field */
	private Boolean deleted;

	/** persistent field */
	private String email;

	/** identifier field */
	private Long id;

	/** persistent field */
	private Set<ReqDoc> lockedReqDocs;

	/** persistent field */
	private String name;

	/** persistent field */
	private String password;

	/** persistent field */
	private Set<ProjectRegistration> projectRegistrations;

	/** persistent field */
	private Boolean sysAdmin;

	/** persistent field */
	private String uid;

	/** nullable persistent field */
	private Integer version;

	/** default constructor */
	/**
	 * 
	 */
	protected User() {
	}

	/** full constructor */
	/**
	 * @param uid
	 * @param password
	 * @param name
	 * @param email
	 * @param sysAdmin
	 */
	public User(String uid, String password, String name, String email,
			Boolean sysAdmin) {
		if ((uid == null) || (password == null)) {
			throw new InstantiationError(
					"Cannot create user becuase the uid and/or password are null");
		}

		this.uid = uid;
		this.password = password;
		this.name = name;
		this.email = email;
		this.sysAdmin = sysAdmin;
	}

	public void addProject(Short projectId, Boolean projectAdmin) {
		final ProjectRegistration project = new ProjectRegistration(projectId,
				projectAdmin, this);

		projectRegistrations.add(project);
	}

	@Override
	public boolean equals(Object other) {
		if ((this == other)) {
			return true;
		}

		if (!(other instanceof User)) {
			return false;
		}

		final User castOther = (User) other;

		return new EqualsBuilder().append(getUid(), castOther.getUid())
				.isEquals();
	}

	@Column(length = 128)
	public String getEmail() {
		return email;
	}

	@Id
	@Column(updatable = false, nullable = false)
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	public Long getId() {
		return id;
	}

	@OneToMany(mappedBy = "lockedBy")
	public Set<ReqDoc> getLockedReqDocs() {
		return lockedReqDocs;
	}

	@Column(length = 64, nullable = false)
	@Basic(optional = false)
	public String getName() {
		return name;
	}

	@Column(length = 32, nullable = false)
	@Basic(optional = false)
	public String getPassword() {
		return password;
	}

	@Transient
	public ProjectRegistration getProjectRegistration(Short projectId) {
		for (final ProjectRegistration projectRegistration : projectRegistrations) {
			if (!projectRegistration.isDeleted()
					&& projectId.equals(projectRegistration.getProjectId())) {
				return projectRegistration;
			}
		}

		return null;
	}

	// FIXME @OneToMany(mappedBy = "user", cascade = CascadeType.ALL)
	@OneToMany(mappedBy = "user", cascade = CascadeType.PERSIST)
	public Set<ProjectRegistration> getProjectRegistrations() {
		return projectRegistrations;
	}

	@Column(length = 32, nullable = false, unique = true)
	@Basic(optional = false)
	public String getUid() {
		return uid;
	}

	@Version
	protected Integer getVersion() {
		return version;
	}

	@Override
	public int hashCode() {
		return new HashCodeBuilder().append(getUid()).toHashCode();
	}

	/**
	 * Answers the question,
	 * "Does the user have administrative privileges on the designated projectId?"
	 * 
	 * @param projectId
	 *            the unique identifier in the Useme system for the projectId.
	 * @return <code>True</code> if the user is privileged, otherwise
	 *         <code>false</code.
	 */
	public Boolean isAdmin(Short projectId) {
		final ProjectRegistration projectRegistration = getProjectRegistration(projectId);

		if (projectRegistration == null) {
			return false;
		}

		return projectRegistration.isProjectAdmin();
	}

	/**
	 * Answers the question,
	 * "Is the user authorized to access the designated projectId?"
	 * 
	 * @param projectId
	 *            the unique identifier in the Useme system for the projectId.
	 * @return <code>True</code> if the user is authorized, otherwise
	 *         <code>false</code.
	 */
	public Boolean isAuthorized(Short projectId) {
		final ProjectRegistration projectRegistration = getProjectRegistration(projectId);

		return projectRegistration != null ? true : false;
	}

	/**
	 * Answers the question, "Has the use been removed from the server?"
	 * 
	 * @param projectId
	 *            the unique identifier in the Useme system for the projectId.
	 * @return <code>True</code> if the user is authorized, otherwise
	 *         <code>false</code.
	 */
	public Boolean isDeleted() {
		return deleted;
	}

	/**
	 * Answers the question,
	 * "Has the user already opened the designated projectId?"
	 * 
	 * @param projectId
	 *            the unique identifier in the Useme system for the projectId.
	 * @return <code>True</code> if the projectId is open, otherwise
	 *         <code>false</code.
	 */
	public Boolean isOpen(Short projectId) {
		final ProjectRegistration projectRegistration = getProjectRegistration(projectId);

		if (projectRegistration == null) {
			return false;
		}

		return projectRegistration.isOpen();
	}

	/**
	 * Answers the question,
	 * "Is the user subscribed to the designated projectId?"
	 * 
	 * @param projectId
	 *            the unique identifier in the Useme system for the projectId.
	 * @return <code>True</code> if the user subscribed, otherwise
	 *         <code>false</code.
	 */
	public Boolean isSubscribed(Short projectId) {
		final ProjectRegistration projectRegistration = getProjectRegistration(projectId);

		if (projectRegistration == null) {
			return false;
		}

		return projectRegistration.isSubscribed();
	}

	public Boolean isSysAdmin() {
		return sysAdmin;
	}

	/**
	 * Open the project.
	 * 
	 * @param projectId
	 *            the project of interest.
	 * @param session
	 * @param hostId
	 *            the id of the host the user is working on
	 * @throws AuthorizationException
	 *             if the analyst is not authorized to open or close the
	 *             project.
	 */
	public void lockReqDoc(final ReqDoc reqdoc, final Boolean hardlock,
			final Boolean softlock) throws LockViolationException {
		assert hardlock != softlock : "hardlock and softlock can not both be set to "
				+ hardlock;
		assert ((hardlock == true) || (softlock == true)) : "hardlock and softlock can not both be set to false";

		reqdoc.lockDoc(this, hardlock, softlock);
		lockedReqDocs.add(reqdoc);
	}

	/**
	 * Revoke the user's authorization to access all projects.
	 */
	public void openProject(Short projectId, UsemeSession session)
			throws AuthorizationException {
		final ProjectRegistration projectRegistration = getProjectRegistration(projectId);

		if (projectRegistration == null) {
			throw new AuthorizationException("User " + uid
					+ " is not authorized on project " + projectId);
		}

		projectRegistration.openProject(session);
	}

	/**
	 * Revoke the user's authorization to access the designated projectId.
	 * 
	 * @param projectId
	 *            the unique identifier in the Useme system for the projectId.
	 */
	public void revokeAllAuthorizations() {
		final Iterator<ProjectRegistration> itor = projectRegistrations
				.iterator();

		while (itor.hasNext()) {
			final ProjectRegistration projectRegistration = itor.next();

			projectRegistration.setDeleted(true);
			projectRegistrations.remove(projectRegistration);
		}
	}

	public void revokeAuthorization(Short projectId) {
		final Iterator<ProjectRegistration> itor = projectRegistrations
				.iterator();

		while (itor.hasNext()) {
			final ProjectRegistration projectRegistration = itor.next();

			if (projectRegistration.getProjectId().equals(projectId)) {
				projectRegistration.setDeleted(true);
				projectRegistrations.remove(projectRegistration);

				return;
			}
		}

		logger.error("Can not revoke authorization because User[" + toString()
				+ "] not registered for Project " + projectId);
	}

	public void setDeleted(Boolean deleted) {
		this.deleted = deleted;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ormf.server.base.services.persistence.domain.IUser#setEmail
	 * (java.lang.String)
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @param id
	 */
	protected void setId(Long id) {
		this.id = id;
	}

	/**
	 * @param lockedReqDocs
	 */
	protected void setLockedReqDocs(Set<ReqDoc> lockedReqDocs) {
		this.lockedReqDocs = lockedReqDocs;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public void setProjectRegistrations(
			Set<ProjectRegistration> projectRegistrations) {
		this.projectRegistrations = projectRegistrations;
	}

	/**
	 * Sets the subscription state of the project.
	 * 
	 * @param projectId
	 *            the project of interest.
	 * @param state
	 *            the state to set.
	 * @throws AuthorizationException
	 * @throws ProjectOpenInAnotherSessionException
	 */
	public void setSubscribed(Short projectId, Boolean state)
			throws AuthorizationException, ProjectOpenInAnotherSessionException {
		final ProjectRegistration projectRegistration = getProjectRegistration(projectId);

		if (projectRegistration == null) {
			throw new AuthorizationException("User " + uid
					+ " is not authorized on project " + projectId);
		}

		if ((state == false) && projectRegistration.isOpen()) {
			final String host = projectRegistration.getSession().getHostId();
			throw new ProjectOpenInAnotherSessionException(
					"The project is open in a session on host " + host);
		}

		projectRegistration.setSubscribed(state);
	}

	public void setSysAdmin(Boolean sysAdmin) {
		this.sysAdmin = sysAdmin;
	}

	/**
	 * @param uid
	 */
	protected void setUid(String uid) {
		this.uid = uid;
	}

	/**
	 * @param version
	 */
	protected void setVersion(Integer version) {
		this.version = version;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("id", getId()).append("uid",
				getUid()).toString();
	}

	public void unlockReqDoc(ReqDoc reqdoc) throws LockViolationException {
		reqdoc.unlockDoc(this);
		lockedReqDocs.remove(reqdoc);
	}

}